# -*- coding: utf-8 -*-

import os
import logging
from . import __title__, __version__
from .base import BaseStructure


FORMATTER = '[+] [%(asctime)s] [%(levelname)s] %(message)s'
logging.basicConfig(format = FORMATTER)
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)


def project(name):
  """create project"""
  # we can use any of the below three variables to make our code compatible with
  # many OS, but we go with the first one which is `self.os_name`
  os_name = os.name # nt or posix
  # platform_name_1 = sys.platform # win32 or linux or darwin or android
  # platform_name_2 = platform.system() # Windows, Darwin, Linux, etc.
  
  if os_name == 'nt' or os_name == 'posix':
  # if platform_name_1 == 'win32' or platform_name_1 == 'linux' or platform_name_1 == 'darwin' or platform_name_1 == 'android':
  # if platform_name_2 == 'Windows' or platform_name_2 == 'Linux' or platform_name_2 == 'Darwin'
    if name.lower() == __title__:
      print()
      LOGGER.error(f'Not allowed to use ({__title__}) package name as project name\n')
      return False
    BaseStructure().dir_tree(name)
  else:
    err_compt = f'{__title__.capitalize()} v{__version__} is not compatible with your OS'
    print()
    LOGGER.error(err_compt)
    print()
    exit()



# -*- coding: utf-8 -*-

import os
import secrets
from werkzeug.utils import secure_filename
from flask import render_template, request, redirect, url_for, flash, send_from_directory, send_file
from flask_login import login_user, current_user, logout_user, fresh_login_required, login_required
from sakyum.utils import footer_style
from sakyum.blueprint import auth
from sakyum.contrib import db, bcrypt
from .models import User
from .forms import LoginForm, ChangePasswordForm, RegisterForm


OS_SEP = os.path.sep # platform-specific path separator (for linux `/`, for windows `\\`)
UPLOAD_FOLDER = os.environ.get('FLASK_UPLOAD_FOLDER')
ORIGIN_PATH = os.environ.get('FLASK_ORIGIN_PATH')
ALLOWED_EXTENSIONS = os.environ.get('FLASK_ALLOWED_EXTENSIONS')


@auth.route('/admin/register/', methods=['POST', 'GET'])
@login_required
def adminRegister():
  form = RegisterForm()
  if form.validate_on_submit():
    username  = form.username.data
    email  = form.email.data
    raw_password2 = form.confirm_password.data
    hashed_password = bcrypt.generate_password_hash(raw_password2).decode('utf-8')
    user = User(username=username, email=email, password=hashed_password)
    db.session.add(user)
    db.session.commit()
    flash('Your created new user, the user is able to log in', 'success')
    return redirect(url_for('auth.adminLogin'))
  context = {
    'head_title': 'admin register',
    'footer_style': footer_style,
    'form': form,
  }
  return render_template('admin_register.html', context=context)
  

@auth.route('/admin/login/', methods=['POST', 'GET'])
def adminLogin():
  if current_user.is_authenticated:
    return redirect(url_for('base.index'))
  form = LoginForm()
  if form.validate_on_submit():
    username = form.username.data
    password = form.password.data
    user = User.query.filter_by(username=username).first()
    if user and bcrypt.check_password_hash(user.password, password):
      login_user(user, remember=True)
      flash('You are now logged in!', 'success')
      next_page = request.args.get('next')
      return redirect(next_page) if next_page else redirect(url_for('admin.index'))
    else:
      flash('Login Unsuccessful. Please check username and password', 'error')
  context = {
    'head_title': 'admin login',
    'footer_style': footer_style,
    'form': form,
  }
  return render_template('admin_login.html', context=context)
  

@auth.route('/admin/change/password/', methods=['POST', 'GET'])
@fresh_login_required
def adminChangePassword():
  """
  The `admin_change_password.html` below is located in the sakyum package (templates/default_page/admin_change_password.html)
  """
  form = ChangePasswordForm()
  if request.method == 'POST':
    old_password = form.old_password.data
    password1 = form.password1.data
    password2 = form.password2.data
    user = User.query.filter_by(username=current_user.username).first()
    if user and bcrypt.check_password_hash(user.password, old_password):
      if password1 == password2:
        hashed_password = bcrypt.generate_password_hash(password2).decode('utf-8')
        user.password = hashed_password
        db.session.commit()
        logout_user()
        flash('Your password has changed!', 'succes')
        return redirect(url_for('auth.adminLogin'))
      else:
        flash('The two password fields didn`t match!', 'error')
    else:
      flash('Cross check your login credentials!', 'error')
  context = {
    'head_title': 'admin change password',
    'footer_style': footer_style,
    'form': form,
  }
  return render_template('admin_change_password.html', context=context)
  

@auth.route('/admin/logout/', methods=['POST', 'GET'])
@login_required
def adminLogout():
  logout_user()
  flash('You logged out!', 'info')
  return redirect(url_for('auth.adminLogin'))
  

def allowed_file(filename):
  return '.' in filename and filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS
  

@auth.route('/profile_image/<path:filename>')
@login_required
def profile_image(filename):
  """
  This function help to show current user profile image, it won't download it
  like the `download_file` function below does
  """
  return send_file(UPLOAD_FOLDER + OS_SEP + filename)
  

@auth.route('/media/<path:filename>')
@login_required
def download_file(filename):
  """
  If we use this to show current user profile image, it won't show instead it will download it,
  so it meant for downloading media file
  """
  return send_from_directory(UPLOAD_FOLDER, filename, as_attachment=True)
  

def picture_name(pic_name):
  random_hex = secrets.token_hex(8)
  _, f_ext = os.path.splitext(pic_name)
  picture_fn = random_hex + f_ext
  new_name = _ + '_' + picture_fn
  return new_name
  

@auth.route('/admin/change_profile_image/', methods=['POST', 'GET'])
@login_required
def changeProfileImage():
  if request.method == 'POST':
    # check if the post request has the file part
    if 'file' not in request.files:
      flash('No file part')
      return redirect(request.url)
    file = request.files['file']
    # If the user does not select a file, the browser submits an
    # empty file without a filename.
    if file.filename == '':
      flash('No selected file')
      return redirect(request.url)
    if file and allowed_file(file.filename):
      filename = secure_filename(file.filename)
      file_name = picture_name(filename)
      file.save(os.path.join(UPLOAD_FOLDER, file_name))
      user = User.query.filter_by(username=current_user.username).first()
      if user:
        if user.user_img != 'default_img.png':
          r = str(ORIGIN_PATH) + OS_SEP + 'media' + OS_SEP + user.user_img
          if os.path.exists(r):
            os.remove(r)
        user.user_img = file_name
        db.session.commit()
      flash('Your profile image has been changed!', 'success')
      return redirect(url_for('base.index')) # it will redirect to the home page
  context = {
    'head_title': 'admin change profile image',
    'footer_style': footer_style,
  }
  return render_template('admin_change_profile_image.html', context=context)



# -*- coding: utf-8 -*-

from flask_sqlalchemy import SQLAlchemy
from flask_login import LoginManager
from flask_bcrypt import Bcrypt
from flask_wtf.csrf import CSRFProtect


db = SQLAlchemy()
bcrypt = Bcrypt()
login_manager = LoginManager()
# To enable CSRF protection globally for Flask, using secret key to securely sign the token
csrf = CSRFProtect()
login_manager.session_protection = 'strong'
login_manager.login_view = 'auth.adminLogin'
login_manager.login_message_category = 'info'
login_manager.login_message = u'You must login, in other to get access to that page'
ext_lst = [db, bcrypt, login_manager, csrf]



# -*- coding: utf-8 -*-

import os
import sys
import shlex
import logging
import argparse
import platform
import subprocess as sp

from pathlib import Path

from .mute import thunder_dummy
from .mute.page import _js
from .mute.page import _css
from .mute.page import _html
from .mute.app import app_admin_dummy
from .mute.app import app_forms_dummy
from .mute.app import app_models_dummy
from .mute.app import app_views_dummy
from .mute.app import app_init_dummy
from .mute.project import pro_init_dummy
from .mute.project import pro_secret_dummy
from .mute.project import pro_config_dummy
from .mute.project import pro_routes_dummy
from .auth.models import User
from .contrib import db, bcrypt
from . import __title__
from . import __version__

"""
  NOTSET    ---  0
  DEBUG     ---  10
  INFO      ---  20
  WARNING   ---  30  (default)
  ERROR     ---  40
  CRITICAL  ---  50
"""

FORMATTER = '[+] [%(asctime)s] [%(levelname)s] %(message)s'
logging.basicConfig(format = FORMATTER)
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)

# used for relative path to default image to copy for a project (only)
ORIGIN = Path(__file__).resolve().parent
OS_SEP = os.path.sep # platform-specific path separator (for linux `/`, for windows `\\`)


class BaseStructure:
  """base structure class"""

  def __init__(self, is_software=True):
    """base structure class initializer"""
    self.is_software = is_software

    # we can use any of the below three variables to make our code compatible with
    # many OS, but we go with the first one which is `self.os_name`
    self.os_name = os.name # nt or posix
    self.platform_name_1 = sys.platform # win32 or linux or darwin or android
    self.platform_name_2 = platform.system() # Windows, Darwin, Linux, etc.

    if self.os_name == 'nt':
    # if self.platform_name_1 == 'win32':
    # if self.platform_name_2 == 'Windows':
      self.fls_cmd = 'echo >'
    elif self.os_name == 'posix':
    # elif self.platform_name_1 == 'linux' or self.platform_name_1 == 'darwin' or self.platform_name_1 == 'android':
    # elif self.platform_name_2 == 'Linux' or self.platform_name_2 == 'Darwin':
      self.fls_cmd = 'touch'
    else:
      err_compt = f'{__title__.capitalize()} v{__version__} is not compatible with your OS'
      print()
      LOGGER.error(err_compt)
      print()
      exit()
    self._exs_first = ['index', 'style']
    self._exs_last = ['.html', '.css', '.js']
    

  def append_exs_to_file(self, fls_name=False, _exs_='.py', name=None):
    """
    append .py extension for files if _exs_ value is '.py' or type is str, else if _exs_ type is list, make list of static files `['index.html', 'index.js', 'style.css']`
    """

    if type(_exs_) == list:
      name, exst = self._exs_first, self._exs_last
      lst = []
      
      for i in exst:
        if i == exst[1]: # css
          idx = name[1]
          lst.append(f'{idx}{i}')
        if i == exst[0] or i == exst[2]: # html & js
          for j in exst:
            if j != exst[1]:
              idx = name[0]
              if f'{idx}{j}' in lst:
                pass
              else:
                lst.append(f'{idx}{j}')
      return lst
    if type(_exs_) == str:
      return [i+f'{_exs_}' for i in fls_name]
      

  def file_content(self, _exs=False, content=None, file_name='index', dir_togo=None, route_go=True):
    """insert content in a file"""
    if _exs:
      with open(f'{file_name}{_exs}', 'w') as pay_fls:
        pay_fls.write(f'{content}')
    else:
      with open(f'{file_name}', 'w') as pay_fls:
        pay_fls.write(f'{content}')
    if route_go:
      os.chdir(dir_togo)


  def instanciate_default_img(self):
    """instanciate user default image"""
    file_path = os.path.join(ORIGIN, 'static', 'default_style', 'media', 'default_img.png')
    with open(file_path, mode='rb') as img_read:
      img_read_data = img_read.read()
    with open('default_img.png', mode='wb') as img_write:
      img_write.write(img_read_data)
      

  def file_opt(self, _dir, tree=True, _here=False, _where=False):
    """
    make dir tree if `tree=True` and get into it, if `_here` or `_where` is equal to True
    """
    if tree:
      if self.os_name == 'nt':
        os.makedirs(_dir, exist_ok=True)
        # The exist_ok=True argument ensures that the function
        # does not raise an exception if the directory already exists.
      elif self.os_name == 'posix':
        sp.run(['mkdir', '-p', _dir])
      else:
        err_compt = f'{__title__.capitalize()} v{__version__} is not compatible with your OS'
        print()
        LOGGER.error(err_compt)
        print()
        exit()
    if _here:
      os.chdir(os.path.join(_here, _dir))
    if _where:
      os.chdir(_where)


  def validateProjectOrAppName(self, name, type_of=False):
    """validate project or app name"""
    if type_of:
      proj_or_app = 'project'
    else:
      proj_or_app = 'app'
    un_accept_char = '@-/\\^+#&*!=%$?.>\'<,"\';:`~|}{][)('
    for i in un_accept_char:
      if i in name:
        name_err = f'[{un_accept_char}] are characters that are not allowed to be in your {proj_or_app} name, we found `{i}` in the name ({name})\n'
        print()
        LOGGER.error(name_err)
        exit()
      

  def into_file(self, fls, fls_cmd, file=None, app_default_dummy=None, is_static_file=False, is_app=False, proj_nm=None):
    """
    create files within current directory of `self.file_opt()`
    is_app: if it is True, that mean it will do operation of making app files,
    else it will make for the entire project
    """
    
    if is_app:
      for _fls in fls:
        app_name = os.getcwd().split(OS_SEP)[-1]
        if self.os_name == 'nt':
          sp.run(shlex.split(f'{fls_cmd} {_fls}'), shell=True)
        elif self.os_name == 'posix':
          sp.run(shlex.split(f'{fls_cmd} {_fls}'))
        else:
          err_compt = f'{__title__.capitalize()} v{__version__} is not compatible with your OS'
          print()
          LOGGER.error(err_compt)
          print()
          exit()
        if is_static_file:
          # building app default files (html, css, js)
          self.file_content(file_name=_fls, content=f'{app_default_dummy}', route_go=False)
        else:
          # building app default files (__init__.py, admin.py, forms.py, models.py, views.py)
          if _fls == '__init__.py':
            self.file_content(file_name=_fls, content=f'# from {__title__} software, your app ({app_name}) {_fls} file\n{app_init_dummy()}', route_go=False)
          elif _fls == 'admin.py':
            self.file_content(file_name=_fls, content=f'# from {__title__} software, your app ({app_name}) {_fls} file\n{app_admin_dummy()}', route_go=False)
          elif _fls == 'forms.py':
            self.file_content(file_name=_fls, content=f'# from {__title__} software, your app ({app_name}) {_fls} file\n{app_forms_dummy()}', route_go=False)
          elif _fls == 'models.py':
            self.file_content(file_name=_fls, content=f'# from {__title__} software, your app ({app_name}) {_fls} file\n{app_models_dummy()}', route_go=False)
          elif _fls == 'views.py':
            self.file_content(file_name=_fls, content=f'# from {__title__} software, your app ({app_name}) {_fls} file\n{app_views_dummy(app_name)}', route_go=False)
    else:
      for _fls in fls:
        if _fls[:-3] == file:
          if self.os_name == 'nt':
            sp.run(shlex.split(f'{fls_cmd} {_fls}'), shell=True)
          elif self.os_name == 'posix':
            sp.run(shlex.split(f'{fls_cmd} {_fls}'))
          else:
            err_compt = f'{__title__.capitalize()} v{__version__} is not compatible with your OS'
            print()
            LOGGER.error(err_compt)
            print()
            exit()
          self.file_content(
            file_name=_fls,content=f'# Your project {_fls} file\n{thunder_dummy(proj_nm)}', route_go=False
            ) # building the run module `thunder.py`
            

  def dir_tree(self, proj_name=None):
    """
    create a directory tree where file will reserved as well as modules too
    """
    self.validateProjectOrAppName(proj_name, type_of=True)
    dirs = [proj_name, f'{proj_name}{OS_SEP}{proj_name}', 'media', 'templates', 'static']
    
    # default files of project sub folder, except `thunder` which is for project base dir
    fls_name = ['__init__', 'config', 'routes', 'secret', 'thunder']
    fls = self.append_exs_to_file(fls_name=fls_name) # appending extensions to files
    _here = os.getcwd() # initial `cwd` where the project was e.g `Desktop`
    
    # check if the project already exist
    if os.path.exists(os.path.join(_here, proj_name)):
      print(f'\nProject ({proj_name}) already exist in this directory\n\t' + os.path.realpath(proj_name))
      LOGGER.info(_here)
      print()
    else:
      # making directories trees and their default files in the loop
      for _dir in dirs:
        if _dir == dirs[0] + OS_SEP + dirs[0]:
          self.file_opt(_dir, _here=_here)
          # create default modules inside project sub dir
          for _fls in fls:
            if _fls[:-3] != 'thunder':
              if self.os_name == 'nt':
                sp.run(shlex.split(f'{self.fls_cmd} {_fls}'), shell=True)
              elif self.os_name == 'posix':
                sp.run(shlex.split(f'{self.fls_cmd} {_fls}'))
              else:
                err_compt = f'{__title__.capitalize()} v{__version__} is not compatible with your OS'
                print()
                LOGGER.error(err_compt)
                print()
                exit()
              # building project default files (__init__.py, config.py, routes.py, secret.py)
              if _fls == '__init__.py':
                self.file_content(file_name=_fls, content=f'# from {__title__} software, your ({proj_name}) project {_fls} file\n{pro_init_dummy()}', route_go=False)
              elif _fls == 'config.py':
                self.file_content(file_name=_fls, content=f'# from {__title__} software, your ({proj_name}) project {_fls} file\n{pro_config_dummy(proj_name)}', route_go=False)
              elif _fls == 'routes.py':
                self.file_content(file_name=_fls, content=f'# from {__title__} software, your ({proj_name}) project {_fls} file\n{pro_routes_dummy(proj_name)}', route_go=False)
              elif _fls == 'secret.py':
                self.file_content(file_name=_fls, content=f'# from {__title__} software, your ({proj_name}) project {_fls} file\n{pro_secret_dummy()}', route_go=False)
          os.chdir(_here)
          

        if _dir == dirs[0]:
          self.file_opt(_dir, _here=_here)
          self.into_file(fls, self.fls_cmd, file='thunder', proj_nm=proj_name) # to maker thunder file
          project_folder = os.getcwd() # base dir path of the project (parent dir)
          
          for static_dir in dirs[3:]: # templates & static
            if static_dir == dirs[3:][0]: # templates
              self.file_opt(static_dir, _here=project_folder) # make templates dir and cd into it
              templates_folder = os.getcwd() # base dir path of templates folder
              
              # make project dir in templates and cd into it 
              self.file_opt(proj_name, _here=templates_folder)
              # create project index.html and back to templates base dir path
              self.file_content(self._exs_last[0], content=f'<!-- @{__title__}, {proj_name} (project) index.html page -->\n'+_html('do_nothing', is_landing=True), dir_togo=templates_folder)

              # make project admin dir in templates and cd into it 
              self.file_opt('admin', _here=templates_folder)
              # create project admin index.html and back to project base dir path
              self.file_content(self._exs_last[0], content=f'<!-- @{__title__}, {proj_name} (project) admin index.html page -->\n'+_html('do_nothing', is_admin=True, project_name=proj_name), dir_togo=project_folder)
              
            if static_dir == dirs[3:][1]: # static
              self.file_opt(static_dir, _here=project_folder) # make static dir and cd into
              static_base_dir = os.getcwd() # base dir path of static folder
              
              # make project static dir and cd into, NB: `os.getcwd()` is base dir of static dir
              self.file_opt(proj_name, _where=os.getcwd()+OS_SEP+proj_name)
              # storing the project static dir path before creating any thing and cd into media
              s_p_dir = os.getcwd()
              self.file_opt('media', _where='media') # make media dir for project
              self.file_opt('do_nothing', tree=False, _where=s_p_dir)
              """
              going back with one step, we pass `do_nothing` as a directory name here, to avoid any error even though it do nothing if we give it a real directory name
              """

              self.file_content(self._exs_last[1], file_name='style', content=f'/* @{__title__}, {proj_name} (project) style.css file */\n'+_css(), route_go=False) # create style.css
              
              # create index.js and back to project static base dir path
              self.file_content(self._exs_last[2], content=f'// @{__title__}, {proj_name} (project) index.js file\n'+_js(proj_name), dir_togo=project_folder)
              
          # make media folder and default image inside it
          self.file_opt(dirs[2], _where=dirs[2])
          self.instanciate_default_img()
          os.chdir(_here)

      print()
      LOGGER.info(f'Project ({proj_name}) created successfully!')
      print()
      

class AppStructure(BaseStructure):
  """base structure class"""
  app_store_name = None # app name
  proj_store_name = None # project name
  
  def app_static_and_template(self, file_dummy, top_comment=False, _dir_=False, file=False, app=False, cmd=False, _here_=False):
    """
    #: _dir_ = 'template or static'
    #: file = ['index.html']
    #: app = app name
    #: cmd = 'touch'
    #: _here_ = # initial `inside project folder` where the project was created
    """
    if top_comment == 'html':
      top_comment = f'<!-- @{__title__}, {app} {file[0]} page -->\n'
    if top_comment == 'css':
      top_comment = f'/* @{__title__}, {app} {file[0]} file */\n'
    if top_comment == 'js':
      top_comment = f'// @{__title__}, {app} {file[0]} file\n'
      
    self.file_opt(_dir_, tree=False, _here=_here_) # back to template dir
    self.file_opt(f'{app}', _where=app) # make app dir inside `template`
    self.into_file(file, cmd, is_static_file=True, app_default_dummy=f'{top_comment}{file_dummy}', is_app=True) # making app default file
    
    
  def dir_tree(self, proj_app_name=None):
    """
    create a directory tree where file will reserved as well as modules too
    """
    dirs = [proj_app_name]
    self.validateProjectOrAppName(proj_app_name)

    app_store_name = proj_app_name # store our app name
    fls_name = ['__init__', 'views', 'models', 'forms', 'admin']
    fls = self.append_exs_to_file(fls_name=fls_name)
    roove_dir = ['templates', 'static', 'static']
    _here_app = os.getcwd()  # initial `inside project folder` (parent) directory
    
    # check if the app already exist
    app_proj_name = _here_app.split(OS_SEP)[-1]
    self.proj_store_name = app_proj_name

    if os.path.exists(os.path.join(_here_app, proj_app_name)):
      print(f'\nApp ({proj_app_name}) already exist in this project ({app_proj_name})\n\t' + os.path.realpath(proj_app_name))
      LOGGER.info(_here_app)
      print()
    else:
      # making directories trees and their default files
      for _dir in dirs:
        if _dir == dirs[0]:
          self.file_opt(_dir, _here=_here_app)
          self.into_file(fls, self.fls_cmd, is_app=True) # to maker app default files
          
      self.app_static_and_template(
        _html(app_store_name, project_name=self.proj_store_name, is_base_app=True), top_comment='html', _dir_=roove_dir[0], file=[self.append_exs_to_file(_exs_=[])[0]], app=proj_app_name, cmd=self.fls_cmd, _here_=_here_app
        )
      self.app_static_and_template(
        _css(), top_comment='css', _dir_=roove_dir[1], file=[self.append_exs_to_file(_exs_=[])[2]], app=proj_app_name, cmd=self.fls_cmd, _here_=_here_app
        )
      self.app_static_and_template(
        _js(proj_app_name), top_comment='js', _dir_=roove_dir[2], file=[self.append_exs_to_file(_exs_=[])[1]], app=proj_app_name, cmd=self.fls_cmd, _here_=_here_app
        )
      self.file_opt('media') # an app media folder
      self.file_opt('do_nothing', tree=False, _where=_here_app) # back to project dir
      print()
      LOGGER.info(f'App ({proj_app_name}) created successfully! in {app_proj_name}')
      print()
      

def app_init(app):
  """initialize app in project"""
  AppStructure().dir_tree(app)
  exit()
  

class Boot:
  """
  boot up project operation, app operation, and the server
  """
  def __init__(self, p=None, d=False, h=None, db=db, model=User, pwd_hash=bcrypt):
    self.p = p # port
    self.d = d # debug
    self.h = h # host
    self.db = db # database
    self.model = model
    self.pwd_hash = pwd_hash

  def run(self):
    """run method for creating app and booting up server"""
    
    error_ref_1 = f'''\n Run the command with one of this positional arguments:\n\tcreate_app   ---   ( for creating an app within your project )\n\tboot  ---   ( for booting up your server )'''

    error_ref_2 = f'''
  create_app:
    usage: create_app [-h] --app  [--debug]

    This create an app in your project

    positional arguments:
                    Put positional argument of `create_app` to create app, app are create inside your project

    optional arguments:
      -h, --help     show this help message and exit
      --app , -a     What is the app name
  create_user:
    usage: create user [-h] [--username] [--email] [--password]

    This create user

    positional arguments:
                        Put positional argument of `create_user` to create user

    options:
      -h, --help        show this help message and exit
      --username , -u   What is the username?
      --email , -e      What is the email?
      --password , -p   What is the password?

  boot:
    usage: boot up server [-h] [--port] [--host] [--debug]

    This boot up the server

    positional arguments:
                    Put positional argument of `boot` to bring server up running

    optional arguments:
      -h, --help     show this help message and exit
      --port , -p    What is the port number?
      --host , -H    What is the host?
      --debug , -d   Do you want debug?'''
    if len(sys.argv) == 1:
      print()
      LOGGER.error(f'please run the module with positional argument and flag if needed\n{error_ref_1}')
      exit()

    if sys.argv[1] == 'create_app':
      # prog is the name of the program, default=sys.argv[0]
      parser = argparse.ArgumentParser(prog='create_app', description='This create an app in your project')
      # metavar make the -help to look cleaan
      parser.add_argument('--app', '-a', required=True, type=str, metavar='', help='What is the app name')
      parser.add_argument(dest='create_app', default='create_app', type=str, metavar='', help='Put positional argument of `create_app` to create app, app are create inside your project')
      args = parser.parse_args()
      the_proj = os.getcwd().split(OS_SEP)[-1]
      
      if args.app.lower() == __title__:
        print()
        LOGGER.error(f'Not allowed to use ({__title__}) package name as an app name\n')
        exit()
      elif args.app == the_proj:
        print()
        LOGGER.error(f'Not allowed to use your ({the_proj}) project name as an app name\n')
        exit()
      app_init(args.app)

    elif sys.argv[1] == 'boot':
      parser = argparse.ArgumentParser(prog='boot up server', description='This boot up the server')
      parser.add_argument('--port', '-p', default=5000, required=False, type=int, metavar='', help='What is the port number?')
      parser.add_argument('--host', '-H', default=self.h, required=False, type=str, metavar='', help='What is the host?')
      parser.add_argument('--debug', '-d', default=False, required=False, type=bool, metavar='', help='Do you want debug?')
      parser.add_argument(dest='boot', default='boot', type=str, metavar='', help='Put positional argument of `boot` to bring server up running')
      args = parser.parse_args()
      self.p = args.port
      self.h = args.host
      if os.environ.get('FLASK_DEBUG') and os.environ.get('FLASK_DEBUG') == '1':
        self.d = True
        LOGGER.info(' * You have set an environment variable of `FLASK_DEBUG` to \'1\'')
      else:
        self.d = args.debug
      
      # LOGGER.info(f'@{__title__} v{__version__} | visit: http://localhost:{args.port} (for development)')
      
    elif sys.argv[1] == 'create_user':
      parser = argparse.ArgumentParser(prog='create user', description='This create user')
      parser.add_argument('--username', '-u', required=False, type=str, metavar='', help='What is the username?')
      parser.add_argument('--email', '-e', required=False, type=str, metavar='', help='What is the email?')
      parser.add_argument('--password', '-p', required=False, type=str, metavar='', help='What is the password?')
      parser.add_argument(dest='create_user', default='create_user', type=str, metavar='', help='Put positional argument of `create_user` to create user')
      args = parser.parse_args()

      from .utils import AuthCredentials
      if args.username != None:
        args_u = False
        usr = args.username
      else:
        args_u = True
        usr = None
      if args.email != None:
        args_e = False
        mail = args.email
      else:
        args_e = True
        mail = None
      if args.password != None:
        args_p = False
        pwd = args.password
      else:
        args_p = True
        pwd = None
        
      auth_class = AuthCredentials(username=usr, email=mail, password=pwd, u_args=args_u, e_args=args_e, p_args=args_p)
      auth_result = auth_class.result
      username = auth_result[0]
      email = auth_result[1]
      raw_password = auth_result[2]

      hashed_password = self.pwd_hash.generate_password_hash(raw_password).decode('utf-8')
      user = self.model(username=username, email=email, password=hashed_password, is_admin=True)
      self.db.session.add(user)
      self.db.session.commit()
      print()
      LOGGER.info(f'One user record added ({username})\n')
      exit()
    else:
      print()
      LOGGER.error(f'use a valid positional argument and flag if needed\n{error_ref_2}')
      exit()



# -*- coding: utf-8 -*-

from . import f1
from . import l1
from . import long_comment


def app_init_dummy():
  return f"""from . import views
"""


def app_views_dummy(app):
  """app is the application name of your project"""
  return f"""from flask import (render_template, Blueprint)
from sakyum.utils import footer_style, template_dir, static_dir
from sakyum.contrib import db
# from .models import <app_models>
# from .forms import <model_form>


{app} = Blueprint('{app}', __name__, template_folder=template_dir(), static_folder=static_dir('{app}'))


@{app}.route('/{app}/', methods=['GET', 'POST'])
def index():
  context = {f1}
    'head_title': '{app}',
    'footer_style': footer_style,
  {l1}
  return render_template('{app}/index.html', context=context)
"""


def app_forms_dummy():
  return f"""from flask_wtf import FlaskForm
from wtforms import StringField, SubmitField, TextAreaField
from wtforms.validators import DataRequired, Length


# write your app forms here!
"""


def app_models_dummy():
  return f"""from datetime import datetime
from sakyum.contrib import db


{long_comment}
when ever you create a model, make sure you import it in your project config.py
file and register it to the admin page in other to see it in admin page
{long_comment}

# write your app model here!
"""

def app_admin_dummy():
  return f"""from flask_login import current_user
from flask import redirect, request, url_for
from flask_admin.contrib.sqla import ModelView


# write your app admin model view here!
"""



# -*- coding: utf-8 -*-

class DB:
  """This class handle mostly database management stuffs"""
  def __init__(self):
    pass

  def make_migrations(self):
    pass

  def migrating_database(self):
    pass

  def update_db(self):
    pass

  def dump_db(self):
    pass

  def cleaned_db(self):
    pass

"""
Command line shortcuts for database:
  db_status
  db_migrate
  db_makemigrations
  db_pulldata
  db_pushdata
  db_cleaned
"""



# -*- coding: utf-8 -*-

from flask_login import current_user
from flask import redirect, request, url_for
from flask_admin.contrib.sqla import ModelView


class UserAdminView(ModelView):
  can_delete = True  # enable model deletion
  can_create = True  # enable model deletion
  can_edit = True  # enable model deletion
  page_size = 50  # the number of entries to display on the list view

  # This is used to list the various columns in the order you want them.
  column_list = ('username', 'email', 'password', 'date_created', 'authenticated', 'is_superuser')
  # Columns that you want to be searchable in the model.
  column_searchable_list = ('username', 'email', 'password', 'date_joined', 'authenticated', 'is_superuser')
  # The column that the view should be sorted with by default (when the view is loaded for
  # the first time). The second parameter True tells flask-admin to sort it in descending order.
  column_default_sort = ('username', True)
  # List of columns that can be used to filter.
  column_filters = ('username',)
  

  def is_accessible(self):
    return current_user.is_authenticated
    
    
  def inaccessible_callback(self, name, **kwargs):
    # redirect to login page if user doesn't have access
    return redirect(url_for('auth.adminLogin', next=request.url))



# -*- coding: utf-8 -*-

from flask import Blueprint, render_template
from flask_admin.contrib.sqla import ModelView
from sakyum.utils import footer_style, template_dir, static_dir

# static_folder: the folder where the Blueprint's static files can be found
# static_url_path: the URL to serve static files from
# template_folder: the folder containing the Blueprint's templates
# url_prefix: the path to prepend to all of the Blueprint's URLs
# subdomain: the subdomain that this Blueprint's routes will match on by default
# url_defaults: a dictionary of default values that this Blueprint's views will receive
# root_path: the Blueprint's root dictionary path, whose default values is obtained from the Blueprint's import

default = Blueprint('default', __name__, template_folder=template_dir(temp_from_pkg='default_page'), static_folder=static_dir('default_style', static_from_pkg=True))
errors = Blueprint('errors', __name__, template_folder=template_dir(temp_from_pkg='default_errors'))
auth = Blueprint('auth', __name__, template_folder=template_dir(temp_from_pkg='default_page'))


"""
The `default` blueprint above is the blueprint that is been used by sakyum for linking
it default pages (css, js, and favicon.ico) files and also it can be use for our project
default html pages (landing page route) that is located in your project route.py file
`<project_name>/route.py` like ` @default.route() `, but instead we use ` @base.route() `
for that `<project_name>/routes.py` as default. We also register it in the project routes.py
file `reg_blueprints` list to make it accessible

The `errors` blueprint above is for error pages, you can overite the error pages by
defining them in your project routes.py file `<project_name>/route.py` just like the
way we did in here down below for some of our default error pages (400, 403, 500, etc)
by giving your desire template file path (correspond to your project templates folder)
for each error page

The `auth` blueprint above is for the `login, logout, register, change_password` and
other default authentication system (route) of your project, which will let you log into admin page
"""


def adminModelRegister(admin, reg_models, db):
  """function that will register a direct model (not model view)"""
  for reg_model in reg_models:
    admin.add_view(ModelView(reg_model, db.session))
    

@errors.app_errorhandler(400)
def error_400(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('400.html', context=context), 400


@errors.app_errorhandler(401)
def error_401(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('401.html', context=context), 401


@errors.app_errorhandler(403)
def error_403(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('403.html', context=context), 403


@errors.app_errorhandler(404)
def error_404(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('404.html', context=context), 404


@errors.app_errorhandler(406)
def error_406(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('406.html', context=context), 406


@errors.app_errorhandler(415)
def error_415(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('415.html', context=context), 415


@errors.app_errorhandler(429)
def error_429(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('429.html', context=context), 429


@errors.app_errorhandler(500)
def error_500(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('500.html', context=context), 500


@errors.app_errorhandler(501)
def error_501(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('501.html', context=context), 501


@errors.app_errorhandler(502)
def error_502(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('502.html', context=context), 502


@errors.app_errorhandler(503)
def error_503(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('503.html', context=context), 503


@errors.app_errorhandler(504)
def error_504(error):
  context = {
    'head_title': 'error page',
    'footer_style': footer_style,
  }
  return render_template('504.html', context=context), 504



# -*- coding: utf-8 -*-

from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField
from wtforms.validators import DataRequired, Length, Email, EqualTo, ValidationError
from .models import User


class RegisterForm(FlaskForm):
  username = StringField('Username', validators=[DataRequired(), Length(min=2, max=30)])
  email = StringField('Email', validators=[DataRequired(), Email()])
  # picture = FileField('Update Profile Picture', validators=[FileAllowed(['jpg', 'png'])])
  password = PasswordField('Password', validators=[DataRequired(), Length(min=6)])
  confirm_password = PasswordField('Confirm Password', validators=[DataRequired(), EqualTo('password'), Length(min=6)])
  
  def validate_username(self, username):
    user = User.query.filter_by(username=username.data).first()
    if user:
      raise ValidationError('That username is taken. Please choose a different one.')
      
  def validate_email(self, email):
    user = User.query.filter_by(email=email.data).first()
    if user:
      raise ValidationError('That email is taken. Please choose a different one.')
      

class LoginForm(FlaskForm):
  username = StringField('Username', validators=[DataRequired(), Length(min=2, max=20)])
  password = PasswordField('Password', validators=[DataRequired(), Length(min=6)])
  
  
class ChangePasswordForm(FlaskForm):
  old_password = PasswordField('Old Password', validators=[DataRequired(), Length(min=6)])
  password1 = PasswordField('Password', validators=[DataRequired(), Length(min=6)])
  password2 = PasswordField('Confirm Password', validators=[DataRequired(), EqualTo('password1'), Length(min=6)])



# -*- coding: utf-8 -*-

from . import f1
from . import l1
from . import f2
from . import l2
from . import __title__
from . import stylePage


def _html(name, is_landing=False, is_admin=False, project_name=False, is_base_app=False):
  if is_landing:
    return f"""{f1}% extends 'default_base.html' %{l1}

{f1}% block head_js %{l1}
  <script src="{f2} url_for('base.static', filename='index.js') {l2}"></script>
{f1}% endblock head_js %{l1}
"""
  if is_admin:
    return f"""{f1}% extends 'admin/master.html' %{l1}
{f1}% block body %{l1}
  <a href="/">Go to {project_name} home page</a>
  <br>
  {f1}% if current_user.is_authenticated %{l1}
    <a href="{f2} url_for('auth.adminLogout') {l2}">logout</a>
    <br>
    <a href="{f2} url_for('auth.adminRegister') {l2}">register</a>
    <br>
    <a href="{f2} url_for('auth.changeProfileImage') {l2}">change image</a>
    <br>
    <a href="{f2} url_for('auth.adminChangePassword') {l2}">change password</a>
  {f1}% else %{l1}
    <a href="{f2} url_for('auth.adminLogin') {l2}">login</a>
  {f1}% endif %{l1}
{f1}% endblock body %{l1}
"""
  if is_base_app:
    page_desc = stylePage(name)
    return f"""{f1}% extends '{project_name}/index.html' %{l1}

{f1}% block head_css %{l1}
  <!-- <link rel="stylesheet" type="text/css" href="{f2} url_for('{name}.static', filename='style.css') {l2}"> -->
{f1}% endblock head_css %{l1}

{f1}% block head_js %{l1}
  <script src="{f2} url_for('{name}.static', filename='index.js') {l2}"></script>
{f1}% endblock head_js %{l1}


{f1}% block short_info %{l1}
  {f1}% if current_user.is_authenticated %{l1}
      <p>Hi {f2} current_user.username {l2}! from your ({name}) project app</p>
    {f1}% else %{l1}
      <p>Your ({name}) app says, please login</p>
    {f1}% endif %{l1}
{f1}% endblock short_info %{l1}


{f1}% block main %{l1}
  <h3><pre>({name}) app
{page_desc[1]}
{page_desc[0]}
{page_desc[1]}</pre></h3>
{f1}% endblock main %{l1}
"""


def _css():
  return f"""
* {f1}
  margin: 0;
  padding: 0;
  box-sizing: border-box;
{l1}

html, body, div, span, object, iframe,
h1, h2, h3, h4, h5, h6, p, blockquote, pre,
abbr, address, cite, code,
del, dfn, em, img, ins, kbd, q, samp,
small, strong, sub, sup, var,
b, i,
dl, dt, dd, ol, ul, li,
fieldset, form, label, legend,
table, caption, tbody, tfoot, thead, tr, th, td,
article, aside, canvas, details, figcaption, figure, 
footer, header, hgroup, menu, nav, section, summary,
time, mark, audio, video {f1}
  outline:0;
  /* font-size:100%; */
  vertical-align:baseline;
  background:transparent;
{l1}

body {f1}
  overflow-x: hidden;
  overflow-y: auto;
  font-size: 15px;
  font-family: "Roboto","Lucida Grande","DejaVu Sans","Bitstream Vera Sans", Times 'Segoe UI', Tahoma, Verdana, sans-serif, serif,Verdana,Arial,sans-serif;
{l1}
"""


def _js(name):
  return f"""function test(){f1}
  alert('I am {__title__} test alert for ({name}) index page')
{l1}
"""



# -*- coding: utf-8 -*-

from sakyum import __title__
from sakyum.utils import Security
from sakyum.utils import stylePage


secret = Security()
secure_app = secret.passcode_salt
f1 = '{'
l1 = '}'
f2 = '{{'
l2 = '}}'
long_comment = '"""'


def thunder_dummy(project):
  return f"""from sakyum import Boot
from {project}.routes import reg_blueprints
from {project}.config import create_app
from {project}.secret import load_env


load_env()
boot = Boot()
if __name__ == '__main__':
  boot.run()


app = create_app(reg_blueprints)
app.run(debug=boot.d, port=boot.p, host=boot.h)
"""



# -*- coding: utf-8 -*-

"""
  =====================
  @ sakyum software
  =====================
      _
    /_  /|   / / |/ /  / /\  /|
     / /_|  /_/  / /  / /  \/ |
  /_/ /  | /  | / /__/ /      |
  
  An extension of flask web framework that erase the complexity of structuring flask project blueprint, packages, and other annoying stuffs
"""


__title__ = 'sakyum'
__version__ = '0.0.10'
__author__ = 'Usman Musa'
__author_email__ = 'usmanmusa1920@gmail.com'
__author_website__ = 'https://usmanmusa1920.github.io'
__repository__ = 'https://github.com/usmanmusa1920/sakyum'
__url__ = 'https://sakyum.readthedocs.io'
__copyright__ = 'Copyright (C) 2022 - 2023 Usman Musa'


from .base import Boot
from .api import project



# -*- coding: utf-8 -*-

import os
import re
import base64
import random
import hashlib
import secrets

from pathlib import Path
from getpass import getpass
from sakyum import __title__
from sakyum import __version__


# relative path to the package folder (sakyum)
REL_PATH = Path(__file__).resolve().parent
OS_SEP = os.path.sep # platform-specific path separator (for linux `/`, for windows `\\`)


def static_dir(app, static_from_pkg=False, os_name: 'nt or posix'=os.name):
  """
  relative path to static files
  if `static_from_pkg` is not false, it will use the directory name that is in the library package which is `static`, else it will use for project `static` directory
  """

  if static_from_pkg:
      return str(REL_PATH) + OS_SEP + 'static' + OS_SEP + app
  return os.getcwd() + OS_SEP + 'static' + OS_SEP + app
  

def template_dir(temp_from_pkg=False, os_name=os.name):
  """
  relative path to html page
  if `temp_from_pkg` is not false, it will use the directory name that is in the library package which is `templates`, else it will use for project `templates` directory
  """

  if temp_from_pkg:
    return str(REL_PATH) + OS_SEP + 'templates' + OS_SEP + temp_from_pkg
  return os.getcwd() + OS_SEP + 'templates'
  

def stylePage(name, version=False):
  """function for styling project/app description default pages"""
  if version:
    # it will style the description in the footer
    desc = '@ ' + name + ' software - v' + version
    desc_center = desc.center(len(desc) + 2)
    border = '=' * len(desc_center)
    return [desc_center, border]

  # it will style the description of default app pages
  desc = 'Your ' + name + ' application default pages'
  desc_center = desc.center(len(desc) + 6)
  border = '=' * len(desc_center)
  return [desc_center, border]
  
  
# Style for sakyum default pages:
    # =============================
    #  @ sakyum software - v0.0.10 
    # =============================
footer_style = stylePage(__title__, version=__version__)


def rem_blueprint(lst_blue=None, rem_blue=None):
  # these are blueprint that we don't want to show on the
  # default page so we are removing them from the list
  # reason is they are not route
  for blue in rem_blue:
    if blue in lst_blue:
      # finding the index of the `blue` item blueprint in the list
      err_index = lst_blue.index(blue)
      # removing it `blue` item from the list using it index number
      lst_blue.pop(err_index)
  blueprints_list = lst_blue
  return blueprints_list


class AuthCredentials:
  """Authenticate new user credentials"""
  def __init__(self, username=None, email=None, password=None, u_args=True, e_args=True, p_args=True):
    self.username = username
    self.email = email
    self.password = password
    self.u_args = u_args
    self.e_args = e_args
    self.p_args = p_args

    if self.u_args:
      while self.username == None or self.username == '' or len(self.username) < 3:
        if self.username == None:
          self.username = input('Enter username: ')
          print()
        elif self.username == '':
          print('username can\'t be empty')
          self.username = input('Enter username: ')
          print()
        elif len((self.username)) < 3:
          print('username must be not less than 3 character')
          self.username = input('Enter username: ')
          print()

    if self.e_args:
      while self.email == None or self.email == '':
        if self.email == None:
          self.email = input('Enter email: ')
          print()
        elif self.email == '':
          print('email can\'t be empty')
          self.email = input('Enter email: ')
          print()

    if self.p_args:
      while self.password == None or self.password == '' or len(self.password) < 6:
        if self.password == None:
          self.password = getpass('Enter password: ')
          print()
        elif self.password == '':
          print('password can\'t be empty')
          self.password = getpass('Enter password: ')
          print()
        elif len((self.password)) < 6:
          print('password must be not less than 6 character')
          self.password = getpass('Enter password: ')
          print()
        
  @property
  def validate_username(self):
    """validate username"""
    # the below while is included to check data validation
    # when user uses flags `-u` or `--username`
    while len(self.username) < 3:
      print('username must be not less than 3 character')
      self.username = input('Enter username: ')
      print()
    return self.username

  @property
  def validate_email(self):
    """email validator"""
    pattern = re.compile(r'^[a-zA-Z0-9-_]+@[a-zA-Z0-9]+\.[a-zA-Z0-9]+')
    if re.match(pattern, self.email):
      return self.email
    while not re.match(pattern, self.email):
      print('Put a valid email: ')
      self.email = input('Enter email: ')
      print()
    return self.email

  @property
  def validate_password(self):
    """validate password"""
    # the below while is included to chech data validation
    # when user uses flags `-p` or `--password`
    while len(self.password) < 6:
      print('password must be not less than 6 character')
      self.password = getpass('Enter password: ')
      print()
    return self.password

  @property
  def result(self):
    """user credentials in a list"""
    auth_list = [self.validate_username, self.validate_email, self.validate_password]
    return auth_list
    

class Security:
  """
  Passcode class for suggesting user passcode iteration,
  generating salt, and creating secure passcode for every user
  
  A reqular expression that matches any character that
  should never appear in base 64 encodings would be:
    [^A-Za-z0-9+/=]
  we follow the base64 pattern of [^A-Za-z0-9+/=] that should never appear in base64, (in reqex)
  """
  
  token_sm_alpha = 'abcdefghijklmnopqrstuvwxyz'
  token_cap_alpha = token_sm_alpha.upper()
  token_num = '0123456789'
  token_char = '/+='
  token_sum = token_sm_alpha + token_cap_alpha + token_num
  
  """
  We times the above variable (token_sum) by 2 (total length is 124),
  so that we will randomly select from it without any restriction,
  since we make the minimum length of the salt to be 32 and the maximum to be 64,
  and also it will randomly select from that range of (32 - 64)
  """

  token_times = token_sum * 2
  token_list = list(token_times)
  random.shuffle(token_list) # shuffling the above list
  token_generate = ''.join(token_list)
  
  def __init__(self, token_generate = token_generate):
    self.token_generate = ''.join(token_generate)
    
  @property
  def passcode_salt(self):
    """
    salting our passcode with this class method
    
    By using the random sample method, where we make:
      population = self.token_generate
      k = random.randint(20, 50)
    """

    salt = ''.join(random.sample(self.token_generate, random.randint(20, 50)))
    return salt # return type is string
    
  def passcode_iteration(self, r_min, r_max, r_step):
    """
    Generating a random iteration. The iterations is not static,
    it is dynamic (every user's iteration is randomly choosen),
    by using the random method of randrange
    """

    itter = random.randrange(r_min, r_max, r_step)
    return itter # return type is integer
    
  def get_hash(self, salt: str, itter: int, passwd: str) -> str:
    """
    generating our key using this class method, and also
    the return type of the key is bytes
    """
    key = hashlib.pbkdf2_hmac(
        'sha256', # The hash digest algorithm for HMAC
        passwd.encode('utf-8'), # Convert the password to bytes
        salt.encode('utf-8'), # Convert the salt to bytes
        itter, # iteration type is integer
        dklen=128 # Get a 128 byte key
    )
    
    """
    Base64 encoding convert the binary data (sequence of byte) into text format,
    to avoid data corruption when transfer via only text channel.
    It is Privacy enhanced Electronic Mail (PEM).
    
    We use ascii to encode the (key)
    """
    
    # encodeing the key, type is string
    b64_encode = base64.b64encode(key).decode('ascii').strip()

    # hashing our b64_encode (the second hash), type is string
    hash_result = hashlib.sha256(str.encode(str(b64_encode))).hexdigest()

    # salt + iteration + hash_result, type is string
    secure_ingredient = '%s%d%s' % (salt, itter, hash_result)
    
    # return types of the list is string all, access it by print(self.get_hash.__annotations__)
    return [hash_result, secure_ingredient]

  def secret(self):
    r = random.randint(32, 52)
    secret = secrets.token_hex(r)
    return secret
    
  def __str__(self):
    return f'Passcode security class'



# -*- coding: utf-8 -*-

from . import f1
from . import l1
from . import secure_app
from . import long_comment


def pro_init_dummy():
  return f"""from .config import create_app


app = create_app()
"""


def pro_secret_dummy():
  return f"""import os
from pathlib import Path


ORIGIN_PATH = Path(__file__).resolve().parent.parent
OS_SEP = os.path.sep # platform-specific path separator (for linux `/`, for windows `\\`)


class Config:
  SECRET_KEY = '{secure_app}'
  # The `SQLALCHEMY_DATABASE_URI` might not be compatible with windows OS,
  # change it to your windows drive like: 'C:\path\to\your\default.db' (if you encounter an error)
  SQLALCHEMY_DATABASE_URI = 'sqlite:///'+str(ORIGIN_PATH)+OS_SEP+'default.db'
  # set optional bootswatch theme
  FLASK_ADMIN_SWATCH = 'cerulean'
  UPLOAD_FOLDER = os.path.join(ORIGIN_PATH, 'media')
  ALLOWED_EXTENSIONS = ('png', 'jpg', 'jpeg')
  MAX_CONTENT_LENGTH = 16 * 1024 * 1024


def load_env():
  os.environ['FLASK_UPLOAD_FOLDER'] = Config.UPLOAD_FOLDER
  os.environ['FLASK_ORIGIN_PATH'] = str(ORIGIN_PATH)
  os.environ['FLASK_ALLOWED_EXTENSIONS'] = str(Config.ALLOWED_EXTENSIONS)
"""


def pro_config_dummy(proj_name):
  return f"""from flask_admin import Admin
from flask import Flask
from sakyum.blueprint import adminModelRegister
from sakyum.contrib import ext_lst, db
from .secret import Config


def create_app(reg_blueprints=False, conf=Config):
  app = Flask(__name__)
  app.config.from_object(conf)
  app.app_context().push()
  for ext in ext_lst:
    ext.init_app(app)


  {long_comment} You will need to import models themselves before issuing `db.create_all` {long_comment}
  from sakyum.auth.models import User
  from sakyum.auth.admin import UserAdminView
  # from <app_name>.models import <app_model>
  # from <app_name>.admin import <admin_model_view>
  db.create_all() # method to create the tables and database
  

  if reg_blueprints:
    for reg_blueprint in reg_blueprints:
      app.register_blueprint(reg_blueprint)


  def admin_runner():
    # Model views allow you to add a dedicated set of admin
    # pages for managing any model in your database
    admin = Admin(app, name='{proj_name}')


    # rgister model to admin direct by passing every model that you
    # want to manage in admin page in the below list (reg_models)
    reg_models = [
      # User,
      # <app_model>,
    ]
    adminModelRegister(admin, reg_models, db)
    # admin model view be here!
    admin.add_view(UserAdminView(User, db.session, name='User', category='User-section'))

  admin_runner()
  return app
"""


def pro_routes_dummy(proj):
  return f"""from flask import (render_template, Blueprint)
from sakyum import blueprint
from sakyum.utils import footer_style, template_dir, static_dir, rem_blueprint
# from <app_name>.views import <app_name>


base = Blueprint('base', __name__, template_folder=template_dir(), static_folder=static_dir('{proj}'))

rem_blue = [blueprint.default, blueprint.errors, blueprint.auth, base]
reg_blueprints = [
  blueprint.default,
  blueprint.errors,
  blueprint.auth,
  base,
  # <app_name>,
]


@base.route('/', methods=['POST', 'GET'])
def index():
  context = {f1}
    'project_name': '{proj}',
    'footer_style': footer_style,
    'blueprints_list': rem_blueprint(lst_blue=reg_blueprints, rem_blue=rem_blue),
  {l1}
  return render_template('{proj}/index.html', context=context)
  
# overwrite (customise) error pages here
"""



# -*- coding: utf-8 -*-

from datetime import datetime
from sakyum.contrib import db, login_manager
from flask_login import UserMixin


@login_manager.user_loader
def load_user(user_id):
  return User.query.get(int(user_id))


class User(db.Model, UserMixin):
  id = db.Column(db.Integer, primary_key=True)
  date_joined = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
  username = db.Column(db.String(20), unique=True, nullable=False)
  user_img = db.Column(db.String(255), default='default_img.png')
  email = db.Column(db.String(120), unique=True, nullable=False)
  password = db.Column(db.String(255), nullable=False)
  authenticated = db.Column(db.Boolean, default=False)
  is_superuser = db.Column(db.Boolean, default=False)
  is_admin = db.Column(db.Boolean, default=False)

  def is_active(self):
    """True, as all users are active."""
    return True

  def get_id(self):
    """Return the user id to satisfy Flask-Login`s requirements."""
    return self.id

  def is_authenticated(self):
    """Return True if the user is authenticated."""
    return self.authenticated

  def is_anonymous(self):
    """False, as anonymous users aren`t supported."""
    return False

  def __repr__(self):
    return f'User(\'{self.username}\', \'{self.email}\')'



# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages


setup(
  name = 'sakyum', # name of the main package (base folder i.e sakyum)
  version = '0.0.10',
  description = 'An extension of flask web framework that erase the complexity of structuring flask project blueprint, packages, and other annoying stuffs',
  long_description = open('README.md').read() + '\n\n' + open('CHANGELOG').read(),
  long_description_content_type='text/markdown',
  python_requires = '>=3.7',
  platforms='any',
  
  url = 'https://sakyum.readthedocs.io',
  download_url = 'https://pypi.org/project/sakyum',
  author = 'Usman Musa',
  author_email = 'usmanmusa1920@gmail.com',
  license = 'MIT',
  classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    # 'Operating System :: POSIX :: Linux',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Topic :: Internet',
    'Topic :: Software Development :: Libraries :: Python Modules',
  ],
  
  # used when people are searching for a module, keywords separated with a space
  keywords = 'sakyum',
  include_package_data = True, # include files listed in MANIFEST.in
  
  # The list of packages(directories) for your library
  packages = find_packages(), # OR packages=['sakyum'] 
  # If your package is a single module, use this instead of 'packages':
  # py_modules=[''] # list of files (modules) that are not in any directory (at the root dir)
  # the libraries it depends on
  
  # List of other python modules which this module depends on. For example RPi.GPIO
  install_requires = [
    'alembic==1.9.4',
    'bcrypt==4.0.1',
    'click==8.1.3',
    'dnspython==2.3.0',
    'email-validator==1.3.1',
    'Flask==2.2.3',
    'Flask-Admin==1.6.0',
    'Flask-Bcrypt==1.0.1',
    'Flask-Login==0.6.2',
    'Flask-SQLAlchemy==3.0.3',
    'Flask-WTF==1.1.1',
    'greenlet==2.0.2',
    'idna==3.4',
    'importlib-metadata==6.0.0',
    'itsdangerous==2.1.2',
    'Jinja2==3.1.2',
    'Mako==1.2.4',
    'MarkupSafe==2.1.2',
    'SQLAlchemy==1.4.45',
    'typing-extensions==4.5.0',
    'Werkzeug==2.2.3',
    'WTForms==3.0.1',
    'zipp==3.13.0',
  ],
  project_urls={
    'Documentation': 'https://sakyum.readthedocs.io',
    'Source': 'https://github.com/usmanmusa1920/sakyum',
  },
)
